### Persefone.jl - a model of agricultural landscapes and ecosystems in Europe.
###
### This file holds the code for the Eurasian Skylark (Alauda arvensis).
###

"""
    Skylark

*Alauda arvensis* is a common and charismatic species of agricultural landscapes.
At the moment, this implementation is still in development.

**Sources:**
    - Bauer, H.-G., Bezzel, E., & Fiedler, W. (Eds.). (2012).
    Das Kompendium der Vögel Mitteleuropas: Ein umfassendes
    Handbuch zu Biologie, Gefährdung und Schutz (Einbändige
    Sonderausg. der 2., vollständig überarb. und erw. Aufl. 2005).
    AULA-Verlag
"""
@species Skylark begin
    
    eggmaturationtime = 11
    eggharvestmortality = 0.9 #???
    eggpredationmortality = 0.1 #???

    migrationdates = ()

    habitats = @habitat((@landcover() == grass || 
                         (@landcover() == agriculture && @cropname() != "maize")) &&
                        @distanceto(forest) > 5)
    
    @initialise(habitats, pairs=true, initfunction=initskylark)
    
    @phase egg begin
        @kill(@trait(eggpredationmortality), "predation")
        @respond harvest @kill(@trait(eggharvestmortality), "harvest")

        #TODO dies if parents die (due to starvation?)

        if @trait(age) == 14
            @setphase(nestling)
        end
    end

    @phase nestling begin
        #TODO
    end

    @phase breeding begin
        #TODO
    end

    @phase nonbreeding begin
        #TODO
    end

    @phase migration begin
        #TODO
    end

    @phase mating begin
        #TODO
    end
end

"""
    initskylark(skylark, model)

Initialise a skylark individual. Selects migration dates and checks if the
bird should currently be on migration.
"""
function initskylark(animal::Animal, model::AgentBasedModel)
    @debug "Added $(animalid(animal)) at $(animal.pos)"
    animal.migrationdates = migrationdates(animal, model)
    leave = animal.migrationdates[1]
    arrive = animal.migrationdates[2]
    m, d = monthday(model.date)
    migrate = (((m < arrive[1]) || (m == arrive[1] && d < arrive[2])) ||
               ((m > leave[1]) || (m == leave[1] && d >= leave[2])))
    if migrate
        returndate = Date(year(model.date), arrive[1], arrive[2])
        model.date != @param(core.startdate) && (returndate += Year(1))
        @migrate(returndate)
    end
    #TODO other stuff?
end

"""
    migrationdates(skylark, model)

Select the dates on which this skylark will leave for / return from its migration,
based on observed migration patterns.
"""
function migrationdates(skylark::Animal, model::AgentBasedModel)
    minleave = skylark.sex == female ? (9, 15) : (10, 1)
    minarrive = skylark.sex == male ? (2, 15) : (3, 1)
    deltaleave = @rand(0:45) #XXX ought to be normally distributed
    deltaarrive = @rand(0:15) #XXX ought to be normally distributed
    leave = monthday(Date(2001, minleave[1], minleave[2]) + Day(deltaleave))
    arrive = monthday(Date(2001, minarrive[1], minarrive[2]) + Day(deltaarrive))
    (leave, arrive)
end
